using Microsoft.EntityFrameworkCore;
using SchoolManagementSidekick;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq.Expressions;
using System.Threading;
using System.Threading.Tasks;

namespace SchoolManagementSidekick
{
    public class PersonService : EntityServiceBase<Person, int>, IPersonService
    {
        private readonly ICurrentUser _currentUser;

        public PersonService(IUnitOfWork unitOfWork, ICurrentUser currentUser) : base(unitOfWork)
        {
            _currentUser = currentUser;
        }

        public async Task<IEnumerable<TEntity>> GetFiltered<TEntity>(Expression<Func<TEntity, bool>> filter, CancellationToken token = default)
            where TEntity : class
        {
            var result = await _unitOfWork.GetRepository<TEntity>().GetQueryable(false)
                .Where(filter)
                .ToListAsync(token);

            return result;
        }

        public async Task<IEnumerable<Person>> FilterByFirstName(string firstName, CancellationToken token = default)
        {
            return await GetFiltered<Person>(p => p.FirstName.Contains(firstName), token);
        }
    }
}
